import zlib

from fastapi import APIRouter, Depends, HTTPException, Path, status
from pydantic import ValidationError

from id_broker_common.kelvin import kelvin_session
from provisioning_plugin.dependencies import check_for_authority_admin, get_school_by_id
from provisioning_plugin.models import NonEmptyStr, NoStarStr, School
from provisioning_plugin.utils import get_pseudonyms_for_providers
from ucsschool.kelvin.client import InvalidRequest, School as KelvinSchool, Session

router = APIRouter(tags=["schools"], dependencies=[Depends(check_for_authority_admin)])


@router.head("/schools/{id}")
@router.get("/schools/{id}", response_model=School)
async def get(
    school_id: NoStarStr = Path(
        ...,
        alias="id",
        description="Unique ID of LDAP object on school authority side.",
        title="Object ID",
    ),
    school_authority: NonEmptyStr = Path(
        ...,
        description="Identifier of the school authority this object originates from.",
        title="School authority ID",
    ),
    school: KelvinSchool = Depends(get_school_by_id),
) -> School:
    try:
        return School.without_prefix(School.from_kelvin_school(school.as_dict()), school_authority)
    except (ValidationError, ValueError) as exc:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"The school {school_id!r} in school_authority {school_authority} is malformed.",
        ) from exc


@router.post("/schools", response_model=School, status_code=201)
async def post(
    school_data: School,
    school_authority: NonEmptyStr = Path(
        ...,
        description="Identifier of the school authority this object originates from.",
        title="School authority ID",
    ),
    session: Session = Depends(kelvin_session),
):
    school_data = school_data.with_prefix(school_data, school_authority)
    # Avoid automatic generation of dc_name by setting it explicitly.
    # The values are not used by the ID-Broker.
    # Schools of the same authority have the same dc_name.
    dc_name = f"dc-{zlib.crc32(school_authority.encode('UTF-8'))}"
    school = KelvinSchool(
        name=school_data.name,
        display_name=school_data.display_name,
        home_share_file_server=dc_name,
        class_share_file_server=dc_name,
        educational_servers=[dc_name],
        udm_properties={"ucsschoolRecordUID": school_data.id, "ucsschoolSourceUID": school_authority},
        session=session,
    )
    async for udm_prop, pseudonym in get_pseudonyms_for_providers(school_data.id, school_authority):
        school.udm_properties[udm_prop] = pseudonym
    try:
        await school.save()
    except InvalidRequest as exc:
        raise HTTPException(status_code=exc.status, detail=exc.reason) from exc
    return School.without_prefix(School.from_kelvin_school(school.as_dict()), school_authority)
